% This function estiamtes 2-D damped sinusoids in noisy measurements. It
% requires no knowledge about the number of modes in the signal. The input
% parameters are:
%
% Y - the data matrix [N1xN2]
% fVecx - Frequency grid for the x:th dimension
% lambda1 - The initial lambda for the Lasso (Good idea to set lambda to
%           the amplitude for the r:th peak in the FFT2D if you expect the
%           number of modes to be <r)
% iterZoom - Number of zooms around the peaks in the Lasso. Good choices are
%            2 or 3.
% t1,t2 - vectors of sample times
%
%  The output parameters are:
%   fEstTx - Frequency estimate for the x:th dimension
%   BetaEstTx - Damping estimate for the x:th dimension
%   FEx/BEx - The estiamtes (FE frequency, BE damping) for the x:th
%             dimension for all iterations
%   deltaF  - Resolution at k:th zoom
%   Zmat    - The latest Lasso estimate
%   freqVx  - The latest zooming grids for the x:th dimension
%
%function [ fEstT1, BetaEstT1,fEstT2, BetaEstT2,FE1,BE1,FE2,BE2,deltaF,Zmat,freqV1,freqV2] =...
%    SEMA2D_New_PumaJS(Y,...
%    fVec1,fVec2,lambda1, iterZoom)


function [ fEstT1, BetaEstT1,fEstT2, BetaEstT2,FE1,BE1,FE2,BE2,deltaF,Zmat,freqV1,freqV2] =...
    SEMA2D_final(Y,...
    fVec1,fVec2,lambda1, iterZoom,t1,t2)


% Save the sizes
%%%%%%%%%%%%%%%%%%%%%%%%%
[N1, N2] = size(Y);     %
Px=length(fVec1);       %
Pz=length(fVec2);       %
zoomGridSize=max(N1,N2);%
%%%%%%%%%%%%%%%%%%%%%%%%%

% Creating initial dictionaries containing beta=0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A1 = createDictionary(t1,fVec1);  %
A2 = createDictionary(t2,fVec2).';%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Preallocating memory
%%%%%%%%%%%%%%
FE1 = {};    %
BE1 = {};    %
FE2 = {};    %
BE2 = {};    %
Zmat_old = 0;%
%%%%%%%%%%%%%%

% Parameters for the ADMM
%%%%%%%%%%%%%%%%%
QUIET=1;        %
noReweights=10; %
rho=10;         %
alpha=1.8;      %
%%%%%%%%%%%%%%%%%

%deltaF = .3.^((1:iterZoom)'-1)*.1;
%deltaF(k) =.3^(k-1)*.1 %Orginal

for k = 1:iterZoom
    
    % Zoom grid size i.e., [deltaF f deltaF]
    %%%%%%%%%%%%%%%%%%%%%%%%%
    deltaF(k) =.3^(k-1)*.05;%
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    % For warm start of ADMM
    %%%%%%%%%%%%%%%%%%%%%%%
    zin = zeros(Px*Pz,1); %
    uin = zin;            %
    %%%%%%%%%%%%%%%%%%%%%%%
    
    % Initial ADMM
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    lambda=ones(1,1)*lambda1;                                                      %
    [z,uin] = ADMM_SEMA2D(A1,A2, Y(:), lambda, rho, alpha,QUIET,zin,uin );%
    zin =z;                                                                        %
    clear Zmat                                                                     %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if noReweights==0 && k==1
        Zmat= reshape(z,Px,Pz);
    end
    
    
    % Rewighted ADMM
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for j=1:noReweights                                                                %
        lambda=lambda1./(abs(z)+0.01);                                                 %
        [z,uin] = ADMM_SEMA2D(A1, A2, Y(:), lambda, rho, alpha,QUIET,zin,uin);%
        zin = z; % For warm start                                                      %
    end                                                                                %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    Zmat= reshape(z,Px,Pz); %Reshaping the estimate to the correct grid structure
    [index, peaks] = peakfinder2D(abs(Zmat)); % Finding peaks
    peaks = peaks(peaks~=0,:); % Returns all 'peaks' even 0-amplitude. Remove all zero-peaks
    FE1{k} =fVec1(index(:,1)); % First esimate
    FE2{k}=fVec2(index(:,2));
    
    % If something is wrong, return previous estimates
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isempty(FE1{k})          %
        Zmat = Zmat_old;        %
        if k == 1               %
            fEstT1 = NaN;       %
            BetaEstT1 = NaN;    %
            fEstT2 = NaN;       %
            BetaEstT2 = NaN;    %
            FE1 = NaN;          %
            BE1 = NaN;          %
            FE2 = NaN;          %
            BE2 = NaN;          %
            deltaF = NaN;       %
            freqV1 = NaN;       %
            freqV2 = NaN;       %
        end                     %
        return                  %
    end                         %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Use a non-linear search to find better estimates of beta and freq.
    % E.g. Puma or NLS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %F = [kron(A2(index(1,2),:).',A1(:,index(1,1))),kron(A2(index(2,2),:).',A1(:,index(2,1)))];
    F = [];
    for kSizeInd = 1:size(index,1)
        Ftemp = kron(A2(index(kSizeInd,2),:).',A1(:,index(kSizeInd,1)));
        F = [F Ftemp];
    end
    z = ((F'*F)\(F'*Y(:)));
    [betaOut, fEstOut]=betaFinder_SEMA2D(Y,F,z,FE1{k},FE2{k},0.001,0.1,Px,Pz,k,t1,t2);
    FE1{k} = fEstOut(1,:)';
    FE2{k} = fEstOut(2,:)';
    BE1{k} = betaOut(1,:)';
    BE2{k} = betaOut(2,:)';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Save the estimates and the old frequency vectors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fEstT1=FE1{k};              %
fEstT2=FE2{k};              %
BetaEstT1=BE1{k};           %
BetaEstT2=BE2{k};           %
freqV1 = fVec1;             %
freqV2 = fVec2;             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Create a new grid which zooms in on the estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if k < iterZoom  % Don't do it in the last iteration...                                                                         %
    [A1, fVec1] = createDictionary(t1,fEstT1,BetaEstT1,deltaF(k),zoomGridSize);                                                 %
    [A2, fVec2] = createDictionary(t2,fEstT2,BetaEstT2,deltaF(k),zoomGridSize);                                                 %
    A2 = A2.';                                                                                                                  %
    Px = size(A1,2);                                                                                                            %
    Pz = size(A2,1);                                                                                                            %
end                                                                                                                             %
Zmat_old = Zmat;                                                                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


end

function [z,u, history,x] = ADMM_SEMA2D(A1,A2, b, lambda, rho, alpha,QUIET,zin,uin)
% ADMM for the SEMA2D algorithm

t_start = tic;


MAX_ITER = 1000;
ABSTOL   = 1e-4;
RELTOL   = 1e-2;
%Data preprocessing

[N1, P1] = size(A1);
[N2, P2] = size(A2.');
[U1,S1,V1] = svd(A1);
[U2,S2,V2] = svd(A2.');
S2=sparse(S2);
S1=sparse(S1);
S2kronS1=sparse(kron(S2,S1));
Villgora=S2kronS1'*S2kronS1;
VecS2kronS1=diag(Villgora)+ones(size(diag(Villgora)))*rho;

B=reshape(b,N1,N2);
% save a matrix-vector multiply
UstarB = U1'*B*conj(U2);
%Ustarb=kron(U2,U1)'*b; %<--- slower way to calculate the above
%norm(Ustarb-UstarB(:))
SigUstarB=(S1')*UstarB*(S2').';


%ADMM solver
x = zeros(size(zin));
if nargin<9
    z=x;
    u=z;
else
    u = uin;
    z=zin;
end

n = P1*P2;


if ~QUIET
    fprintf('%3s\t%10s\t%10s\t%10s\t%10s\t%10s\n', 'iter', ...
        'r norm', 'eps pri', 's norm', 'eps dual', 'objective');
end

for k = 1:MAX_ITER
    
    % x-update
    Z=reshape(z,P1,P2);
    U=reshape(u,P1,P2);
    Q=SigUstarB+rho*V1'*(Z-U)*(V2').';
    qdelat=1./VecS2kronS1.*Q(:);
    Qdelat=reshape(qdelat,P1,P2);
    X=V1*Qdelat*(V2).';
    x=X(:);
    
    
    % z-update with relaxation
    zold = z;
    x_hat = alpha*x + (1 - alpha)*zold;
    z = shrinkage(x_hat + u, lambda/rho);
    
    % u-update
    u = u + (x_hat - z);
    
    history.r_norm(k)  = norm(x - z);
    history.s_norm(k)  = norm(-rho*(z - zold));
    
    history.eps_pri(k) = sqrt(n)*ABSTOL + RELTOL*max(norm(x), norm(-z));
    history.eps_dual(k)= sqrt(n)*ABSTOL + RELTOL*norm(rho*u);
    
    if ~QUIET
        fprintf('%3d\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.2f\n', k, ...
            history.r_norm(k), history.eps_pri(k), ...
            history.s_norm(k), history.eps_dual(k), history.objval(k));
    end
    
    if (history.r_norm(k) < history.eps_pri(k) && ...
            history.s_norm(k) < history.eps_dual(k))
        break;
    end
    
end

if ~QUIET
    toc(t_start);
end
end

function p = objective(A, b, lambda, x, z)
p = ( 1/2*sum(abs(A*x - b).^2) + sum(lambda'*abs(z)) );
end

function z = shrinkage(x, kappa)
z = max(0,abs(x)-kappa).*x./abs(x);
end

function [L U] = factor(A, rho)
[m, n] = size(A);
if ( m >= n )    % if skinny
    L = chol( A'*A + rho*speye(n), 'lower' );
else            % if fat
    L = chol( speye(m) + 1/rho*(A*A'), 'lower' );
end

% force matlab to recognize the upper / lower triangular structure
L = sparse(L);
U = sparse(L');
end


function [betaEst, fEst]=betaFinder_SEMA2D(Y,F,z,FE1,FE2,betamin,betamax,Px,Pz,k,time1,time2)
if isempty(FE1)
    BE1out = [];
    BE2out = [];
    FE1out = [];
    FE2out = [];
    return
end
if iscell(FE1)
    FE1 = cell2mat(FE1);
end

y=Y(:);

fEst = [FE1;FE2];
betaEst = zeros(size(fEst));
for kSnurr = 1:1
    [fEst ,F,z ] = NLSforSEMA2D(y,fEst,betaEst,time1,time2,F,z);
    [ betaEst,F,z ] = NLSforSEMS2D_Damp(y,fEst,betaEst,time1,time2,F,z);
    fEst;
    
end

end
function [ finalEst,Fnew,znew ] = NLSforSEMA2D(y,fEst,betaEst,time1,time2,F,z)
%DONLS_3D Summary of this function goes here
%   Detailed explanation goes here
nbrOfPeaks = size(fEst,2);
P = 100;


residual = y- F*z;
Fnew = [];
znew = Fnew;
outerMax=6;

for iPeak = 1:nbrOfPeaks
    y = residual + F(:,iPeak)*z(iPeak);
    deltaF = .05;
    for iOuter = 1:outerMax
        
        
        %y_rest  = y-A(:,1:end ~= nbrOfPeaks)*ampEst(1:end ~= nbrOfPeaks);
        diffVec = linspace(-deltaF,deltaF,P);
        deltaF = 6*abs(diffVec(2)-diffVec(1));
        f1vec = fEst(1,iPeak)+diffVec;
        f2vec = fEst(2,iPeak)+diffVec;
        f1vec(f1vec>1)=f1vec(f1vec>1)-1;
        f1vec(f1vec<0)=f1vec(f1vec<0)+1;
        f2vec(f2vec>1)=f2vec(f2vec>1)-1;
        f2vec(f2vec<0)=f2vec(f2vec<0)+1;
        
        D1 = exp(time1*(2i*pi*f1vec-betaEst(1,iPeak)));
        D2 = exp(2i*pi*fEst(2,iPeak)*time2-betaEst(2,iPeak)*time2);
        res = zeros(P,1);
        for i = 1:P
            D = kron(D2,D1(:,i));
            res(i) = norm(y-D*((D'*D)\(D'*y)))^2;
        end
        [~, locs] = min(res);
        fEst(1,iPeak)=f1vec(locs);
        
        D1 = exp(2i*pi*fEst(1,iPeak)*time1-betaEst(1,iPeak)*time1);
        D2 = exp(time2*(2i*pi*f2vec-betaEst(2,iPeak)));
        res = zeros(P,1);
        for i = 1:P
            D = kron(D2(:,i),D1);
            res(i) = norm(y-D*((D'*D)\(D'*y)))^2;
        end
        [~, locs] = min(res);
        fEst(2,iPeak)=f2vec(locs);
        D = kron(D2(:,locs),D1);
        
        % F(:,iPeak) = D;
        %z(iPeak) = ((D'*D)\(D'*y));
        if iOuter == outerMax
            D = D/norm(D);
            Fnew = [Fnew D];
            znew = [znew ;((D'*D)\(D'*y))];
        end
    end
    residual = residual - D*((D'*D)\(D'*y));
end
finalEst  = fEst;
end

function [ dampEst,Fnew,znew ] = NLSforSEMS2D_Damp(y,fEst,dampEst,time1,time2,F,z)
%DONLS_3D Summary of this function goes here
%   Detailed explanation goes here
nbrOfPeaks = size(fEst,2);
P = 100;
dampEst1 = dampEst(1,:);
dampEst2 = dampEst(2,:);
residual = y- F*z;
outerMax = 5;
Fnew = [];
znew = [];
for iPeak = 1:nbrOfPeaks
    %y = residual + F(:,iPeak)*z(iPeak);
    for iOuter = 1:outerMax
        
        %y_rest  = y-A(:,1:end ~= nbrOfPeaks)*ampEst(1:end ~= nbrOfPeaks);
        if iOuter==1
            damp1vec = linspace(0,0.1,P);
            damp2vec = linspace(0,0.1,P);
            dB = 10 * abs(damp1vec(2)-damp1vec(1));
        else
            differ = linspace(-dB,dB,P);
            damp1vec = dampEst1(iPeak)+differ;
            damp2vec = dampEst2(iPeak)+differ;
            dB = 10 * abs(damp1vec(2)-damp1vec(1));
        end
        for inner = 1:2
            D1 = exp(time1*(2i*pi*fEst(1,iPeak)-damp1vec));
            D2 = exp(2i*pi*fEst(2,iPeak)*time2-dampEst2(iPeak)*time2);
            res = zeros(P,1);
            for i = 1:P
                D = kron(D2,D1(:,i));
                res(i) = norm(y-D*((D'*D)\(D'*y)))^2;
            end
            [~, locs] = min(res);
            dampEst1(iPeak)=damp1vec(locs);
            
            D1 = exp(2i*pi*fEst(1,iPeak)*time1-dampEst1(iPeak)*time1);
            D2 = exp(time2*(2i*pi*fEst(2,iPeak)-damp2vec));
            res = zeros(P,1);
            for i = 1:P
                D = kron(D2(:,i),D1);
                res(i) = norm(y-D*((D'*D)\(D'*y)))^2;
            end
            [~, locs] = min(res);
            dampEst2(iPeak)=damp2vec(locs);
            D = kron(D2(:,locs),D1);
            residual = residual - D*((D'*D)\(D'*y));
        end
        if iOuter == outerMax
            Fnew = [Fnew D];
            znew = [znew; ((D'*D)\(D'*y))];
        end
    end
end
dampEst  = [dampEst1;dampEst2];
end

function [A, fVec] = createDictionary(time,fVec,beta,fDelta,gridSize)


N = length(time);
if nargin == 2
    A = zeros(N,length(fVec));
    for  kk = 1:length(fVec)
        A(:,kk) = exp(2i*pi*fVec(kk)*time(:));
        A(:,kk) =  A(:,kk)/norm(A(:,kk));
    end
elseif nargin>2
    K = length(beta);
    %A = zeros(N,gridSize*K);
    A = [];
    fV = [];
    for k = 1:K
        f = fVec(k)+linspace(-fDelta,fDelta,gridSize);
        f(f>1) = f(f>1)-1;
        f(f<0) = f(f<0)+1;
        B = zeros(N,gridSize);
        for kk = 1:gridSize
            B(:,kk) = exp(2i*pi*f(kk)*time(:)-beta(k)*time(:));
            B(:,kk) = B(:,kk)/norm(B(:,kk));
        end
        A = [A B];
        fV = [fV f];
    end
    fVec = fV;
else
    display('Not enough or too many input arguments');
end
end
% This function returns the indices of the peaks in the matrix X as
% well as the index matrix such that peaks = X(index).
% The peaks are sorted in descending order and is returned in peaks
% as [row_index, column_index, height] (one row of peaks, a N*3-matrix
% where N is the number of peaks in X). A peak is defined as an element
% where all surrounding elements are smaller than or equal to the
% element itself (eight surrounding elements).
%
% When real elements this function consider negative values. When the
% elements are complex, size is measured as the absolute value of the
% elements.
%
% The function now returns the nbrOfPeaks largest peaks. If omitted, it
% returns all peaks. (Erik Gudmundson, Aug. 2007)
%
% function [peaks, index] = peakfinder(X,nbrOfPeaks)
function [index, ppeaks] = peakfinder2D(X,nbrOfPeaks)
if size(X,3) == 1
    Y = ones(size(X) + [2 2]) * -Inf; % create matrix of -Inf
    Y(2:size(Y, 1)-1, 2:size(Y, 2)-1) = X; % Y is now X surrounded by -Inf
    row = 1;
    ppeaks = [];
    
    
    for j=2:size(Y, 1)-1
        for k = 2:size(Y, 2)-1
            if (Y(j, k) > max([Y(j-1,k-1), Y(j-1, k), Y(j-1, k+1), ...
                    Y(j, k-1), Y(j, k+1), Y(j+1, k-1), Y(j+1, k), ...
                    Y(j+1, k+1)]))
                ppeaks(row, :) = [j-1, k-1, Y(j,k)];
                row = row + 1;
            end
        end
    end
else
    disp('The matrix is not in 2D')
    return
end


[peaksS,index ] = sortrows(ppeaks, 3);
if nargin == 1
    nbrOfPeaks = length(index);
end
peaksS = flipud(peaksS);
if size(peaksS,1)<nbrOfPeaks
    index = flipud(index);
    return
end

ppeaks = peaksS(1:nbrOfPeaks,3);
index = peaksS(1:nbrOfPeaks,1:2);
end