% This is a script containing a simple example showing how to use
% the SEMA algorithm on a 2-D signal. Any question about SEMA can be sent
% to js@maths.lth.se.

clearvars
close all
clc


%%%%%%%%%% Settings for the signal %%%%%%%%%%
N1 = 33; % Number of samples in the first dimension
N2 = 31; % Number of samples in the second dimension
SNR = 20; % SNR-level (dB) for the signal
nbrOfPeaks = 3; % True number of modes in the signal
alpha = 1*ones(nbrOfPeaks,1).*exp(randn(nbrOfPeaks,1)*2i*pi); % Create complex amplitude

tTot1 = linspace(0,N1-1,100*N1); % Non-uniform
tTot2 = linspace(0,N2-1,N2); % Uniform
t1 = sort(tTot1(randperm(100*N1,N1)))';
t2 = sort(tTot2(randperm(N2,N2)))';

%%% If only uniformly sampling in both dimensions %%%
 t1 = (0:N1-1)';                                   %    
 t2 = (0:N2-1)';                                   %    
%%%                                               %%%

tmp=rand(nbrOfPeaks,2); % Randomly select frequencies
% Make sure they are reasonalby separated
while min(diff(sort(tmp(:,1))))<1/N1 || min(diff(sort(tmp(:,2))))<1/N1 || min(min(tmp))<1/N1 || max(max(tmp))>(1-1/N1)
    tmp= rand(nbrOfPeaks,2);
end
freqs=tmp; % Save the true frequencies
beta1=[0.05 0.02 0.01];% Set the dampings in the first dimension
beta2=[0.01 0.04 0.01];% Set the dampings in the second dimension
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Settings for the algorithm %%%%%%%%%%
noFreqQrid=250;% Number of grid points in the frequency grid
fVec1 = linspace(0,1-1/noFreqQrid,noFreqQrid); % Frequency grid for the first dimension
fVec2 = fVec1;% Frequency grid for the second dimension
BetaMax = .1; % Maximum beta value in the grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%% Simulate data %%%%%%%%%
Y = zeros(N1,N2);
Yfinal=Y;
for ii = 1:nbrOfPeaks
    for kk = 1:N1
        for jj = 1:N2
            Y(kk,jj)  =  alpha(ii)*exp((2i*pi*freqs(ii,1)-beta1(ii))*t1(kk)+(2i*pi*freqs(ii,2)-beta2(ii))*t2(jj));
        end
    end
    Yfinal=Yfinal+alpha(ii)*Y/norm(Y(:));
end
Y=Yfinal; % Original signal
sigma = sqrt(norm(Y(:))^2/length(Y(:))*10^(-SNR/10));
noise = sigma*(randn(N1,N2)+1i*randn(N1,N2))/sqrt(2);
Y = Y + noise; % Add noise
Y = 10*Y; % Scale the data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Create dictionaries %%%%%%%%%%
A1 = createDictionary(t1,fVec1); % Create dictionary first dimension
A2 = createDictionary(t2,fVec2).'; %  Create dictionary second dimension

Px = length(fVec1); % Length of the frequency grid first dimension
Pz = length(fVec2); % Length of the frequency grid second dimension
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Select \lambda %%%%%%%%%%
lambda_new = .5*max(abs(kron(A2.',A1)'*Y(:))); % Examples to how to set \lambda: If uniformly sampled data, set \lambda to the 10th largest
% peak in the periodogram or (works for any kind of sampling) set 
% \lambda=c*||F'*y||_inf, where c \in [0,.9].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%% Estimation %%%%%%%%%%%
iterMax = 1; % Number of iteration in SEMA
tic
[ frqEst1, BetaEst1,frqEst2, BetaEst2] =SEMA2D_final(Y,...
    fVec1,fVec2,lambda_new, iterMax,t1,t2);
timeGone = toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Display results %%%%%%%%%%
disp(['Time elapsed ' num2str(timeGone) ' s'])
disp('True frequencies')
[sort(freqs(:,1)) sort(freqs(:,2))]
disp('Estimated frequencies')
[sort(frqEst1) sort(frqEst2)]
disp('True damping')
[sort(beta1)' sort(beta2)']
disp('Estimated damping')
[sort(BetaEst1) sort(BetaEst2)]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
